    function [ phi_opt, coh, coh_avg, coh_vec, coh_avg_vec ] = opt_measurements_mat( phi, psi, mue_init, beta, err_par, eps_par, L, K )

    
%This function optimizes a given measurement matrix relative to a given mixing matrix.
%
%Input parameters:
%phi - measurement matrix
%psi - mixing matrix
%mue_init - initial value of mue^t (see step 3 of the algorithm in our paper)
%beta - parameter used to calculate the step size of FISTA. 
%Should satisfy: 0<beta<=1
%err_par - e_tilda_par in our paper. Should satisfy: 0<e_tilda_par<1
%eps_par - epsilon_par in our paper. Should satisfy: 0<epsilon_par<1
%L - number of iterations of the external loop.
%K - number of iterations of the internal loop.
%
%Output parameters:
%phi_opt - optimized matrix
%coh - mutual coherence of phi_opt * psi
%coh_avg - average mutual coherence of phi_opt * psi
%coh_vec - coh at the end of each external iteration.
%coh_avg_vec - coh_avg at the end of each external iteration.
% 
% You are very welcome to use this code. For this, clearly acknowledge 
% the source of this code, and cite the paper that describes the algorithm:
% M. Mordechay and Y. Y. Schechner, "Matrix optimization for poisson compressed
% sensing," in Proceedings of IEEE GlobalSIP, December 2014, pp.684-688.
%
% (c) Copyright Moran Mordechay 2014. The matlab code is available for 
% non-commercial use and exploration.  For commercial use contact the
% author. The author is not liable for any damages or loss that might be 
% caused by use or connection to this code. 


m = size(psi, 2);
p = size(phi, 1);
inds = eye(m)==1;
psi_orig = psi;

coh_vec = zeros(1, L);
coh_avg_vec = zeros(1, L);
i = 1;
error = 0;
mue = mue_init;
mue_b = 0;

while ( i < L+1 ) 
    H = (psi' * phi') * (phi * psi);
    vec = H(inds);
    norms = sqrt(vec);
    norms_mat = norms * norms';
    H = H ./ norms_mat;
    inds_pos = H >= mue;
    inds_neg = H < -1*mue;
    H(inds_pos) = mue;
    H(inds_neg) = -1*mue;
    if (mue_b>0)
        inds_pos = (H>=0) & (H<mue_b);
        inds_neg = (H<0) & (H>-1*mue_b);
        H(inds_pos) = mue_b;
        H(inds_neg) = -1*mue_b;
    end
    H(inds) = 1;
    [ mue, mue_b, error] = calc_mue(H, m, p, mue, error, err_par, eps_par);
    F = find_tight_sqrt( H, m, p, inds );
    
    norms_rep = repmat(norms', size(psi, 1), 1);
    psi = psi ./ norms_rep;
    
    omega = eye(p);
    lip = 2 * norm(psi*psi', 2);
    eta = beta / lip;
    
    j = 0;
    while (j < K)
        B = omega * F;
        phi = fista(phi, psi, B, eta);
        [X,~,Z] = svd(phi * psi * F');
        X = real(X);
        Z = real(Z);
        omega = X * Z';
        j = j+1;
    end
    [ coh, coh_avg ] = calc_coherence( phi, psi_orig );
    coh_vec(i) = coh;
    coh_avg_vec(i) = coh_avg;
    i
    i = i+1;
end

phi_opt = phi;
[ coh, coh_avg ] = calc_coherence( phi, psi_orig );

end

